<?php

namespace Drupal\sodium\Plugin\EncryptionMethod;

use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\encrypt\Attribute\EncryptionMethod;
use Drupal\encrypt\EncryptionMethodInterface;
use Drupal\encrypt\Exception\EncryptException;
use Drupal\encrypt\Plugin\EncryptionMethod\EncryptionMethodBase;
use ParagonIE\Halite\Alerts\HaliteAlert;
use ParagonIE\Halite\Alerts\InvalidKey;
use ParagonIE\Halite\Symmetric\Crypto;
use ParagonIE\Halite\Symmetric\EncryptionKey;
use ParagonIE\HiddenString\HiddenString;

/**
 * Adds an encryption method that uses Libsodium for cryptographic operations.
 *
 * @EncryptionMethod(
 *   id = "sodium",
 *   title = @Translation("Sodium"),
 *   description = "Uses Libsodium for cryptographic operations.",
 *   key_type = {"encryption"}
 * )
 */
#[EncryptionMethod(
  id: "sodium",
  title: new TranslatableMarkup("Sodium"),
  description: new TranslatableMarkup("Uses Libsodium for cryptographic operations."),
  key_type: ["encryption"]
)]
class SodiumEncryptionMethod extends EncryptionMethodBase implements EncryptionMethodInterface {

  const CLASS_TO_CONFIRM_HALITE_INSTALLATION = '\ParagonIE\Halite\Symmetric\Crypto';

  /**
   * {@inheritdoc}
   */
  public function checkDependencies($text = NULL, $key = NULL): array {
    $errors = [];

    if (!class_exists(self::CLASS_TO_CONFIRM_HALITE_INSTALLATION)) {
      $errors[] = $this->t('Sodium requires the Halite PHP library.');
    }

    return $errors;
  }

  /**
   * {@inheritdoc}
   */
  public function encrypt($text, $key): string {
    // Create the key object.
    try {
      $key_hidden = new HiddenString($key);
      $encryption_key = new EncryptionKey($key_hidden);
    }
    catch (InvalidKey $e) {
      throw new EncryptException($e);
    }

    // Encrypt the data.
    try {
      $text_hidden = new HiddenString($text);
      $encrypted_data = Crypto::encrypt($text_hidden, $encryption_key, TRUE);
    }
    catch (HaliteAlert $e) {
      throw new EncryptException($e);
    }

    return $encrypted_data;
  }

  /**
   * {@inheritdoc}
   */
  public function decrypt($text, $key): string {
    // Create the key object.
    try {
      $key_hidden = new HiddenString($key);
      $encryption_key = new EncryptionKey($key_hidden);
    }
    catch (InvalidKey $e) {
      throw new EncryptException($e);
    }

    // Decrypt the data.
    try {
      $decrypted_data = Crypto::decrypt($text, $encryption_key, TRUE);
      $decrypted_data = $decrypted_data->getString();
    }
    catch (HaliteAlert $e) {
      throw new EncryptException($e);
    }

    return $decrypted_data;
  }

}
